local cached_itemids = {}

function get_itemid(name)
	local id = cached_itemids[name]
	if not id then
		local data = get_item_data_by_name(name)
-- 		local data = kolproxy_log_time_interval("get_item_data_by_name: " .. name, function() return get_item_data_by_name(name) end)
		if not data or not data.id then
			error("No itemid found for item: " .. tostring(name))
		end
		id = tonumber(data.id)
		cached_itemids[name] = id
	end
	return id
end

function intercept_warning(warning)
	if not warning.id then
		error "No warning id!"
	end
	if session["warning-" .. warning.id] then return end
	local head = [[<head><script type="text/javascript" src="http://images.kingdomofloathing.com/scripts/jquery-1.3.1.min.js"></script>
<script type="text/javascript">
	function disable_warning(link) {
		var pwd = ']] .. session.pwd .. [['
		var stateset = 'session'
		var name = 'warning-]] .. warning.id .. [['
		var value = 'skip'
		$.post('custom-settings', { pwd: pwd, action: 'set state', name: name, stateset: stateset, value: value, ajax: 1 }, function (res) {
			link.style.color = 'gray'
			link.innerHTML = '(Warning disabled until you log out)'
		});
	}
</script>
<script>top.charpane.location = "charpane.php"</script></head>]]
	local extratext = ""
	if not warning.norepeat then
		extratext = [[<a href="]]..raw_make_href(requestpath, parse_params_raw(input_params))..[[">Repeat request</a>]]
	end
	text = [[<html>]] .. head .. [[
<p>]] .. (warning.customwarningprefix or "Warning: ") .. warning.message .. [[</p><p>]] ..
[[<a href="#" onclick="javascript:disable_warning(this); return false;" style="color: ]] .. (warning.customdisablecolor or "darkorange") .. [[;">]] .. (warning.customdisablemsg or "Disable this warning") .. [[</a>]] ..
[[</p><p>]] .. extratext .. [[</p><body></body></html>]]
	return text, "/kolproxy-warning"
end


local after_pageload_cache = {}

function reset_pageload_cache()
	after_pageload_cache = {}
end

function get_cached_function(f)
-- 	print("get_cached_function", f)
	local v = after_pageload_cache[f]
	if v then
-- 		print("cached f")
		return v
	else
-- 		print("calling raw f")
		v = f()
		after_pageload_cache[f] = v
		return v
	end
end

function get_cached_item(name, f)
	local v = after_pageload_cache[name]
	if v then
		return v
	else
		v = f()
		after_pageload_cache[name] = v
		return v
	end
end

raw_submit_page = submit_page

function do_submit_page(t, url, params)
	kolproxy_debug_print("> do_submit_page(" .. tostring(url) .. ")\n" .. debug.traceback(""))
	after_pageload_cache = {}
	local pt, pturl = nil, nil
	if params then
		local tbl = {}
		for a, b in pairs(params) do
			if type(b) == "string" then
				table.insert(tbl, { key = a, value = b })
			elseif type(b) == "number" then
				table.insert(tbl, { key = a, value = tostring(b) })
			else
				error("Unknown submit_page value type: " .. type(b))
			end
		end
		pt, pturl = raw_submit_page(t, url, tbl)
	else
		pt, pturl = raw_submit_page(t, url, nil)
	end
	kolproxy_debug_print("< do_submit_page(" .. tostring(url) .. ")")
	return pt, pturl
end

function get_page(url, params) return do_submit_page("GET", url, params) end

function post_page(url, params) return do_submit_page("POST", url, params) end



raw_async_submit_page = async_submit_page

-- TODO: improve async and after_pageload_cache interaction, after_pageload_cache should be cleared after every completed pageload/statuschange!
function do_async_submit_page(t, url, params)
	kolproxy_debug_print("> do_async_submit_page()\n" .. debug.traceback(""))
	after_pageload_cache = {}
	local pt, pturl, tbl = nil, nil, nil
	if params then
		tbl = {}
		for a, b in pairs(params) do
			if type(b) == "string" then
				table.insert(tbl, { key = a, value = b })
			elseif type(b) == "number" then
				table.insert(tbl, { key = a, value = tostring(b) })
			else
				error("Unknown async_submit_page value type: " .. type(b))
			end
		end
	end
	kolproxy_debug_print("< do_async_submit_page()")
	return raw_async_submit_page(t, url, tbl)
end

function async_get_page(url, params) return do_async_submit_page("GET", url, params) end

function async_post_page(url, params) return do_async_submit_page("POST", url, params) end



raw_make_href = make_href

function do_make_href(url, params)
	if params then
		local tbl = {}
		for a, b in pairs(params) do
			if type(b) == "string" then
				table.insert(tbl, { key = a, value = b })
			elseif type(b) == "number" then
				table.insert(tbl, { key = a, value = tostring(b) })
			else
				error("Unknown make_href value type: " .. type(b))
			end
		end
		return raw_make_href(url, tbl)
	else
		return raw_make_href(url, nil)
	end
end

make_href = do_make_href


function run_file_with_environment(filename, orgenv, f_notfound)
	-- TODO: let scripts export functions
-- 	local env = {}
-- 	for a, b in pairs(orgenv) do
-- 		env[a] = b
-- 	end
	local env = orgenv
	if not orgenv.add_printer_raw then
		orgenv.add_printer_raw = orgenv.add_printer
	end
	function env.add_printer(file, func)
		orgenv.add_printer_raw(file, func, filename)
	end
	if not orgenv.add_processor_raw then
		orgenv.add_processor_raw = orgenv.add_processor
	end
	function env.add_processor(file, func)
		orgenv.add_processor_raw(file, func, filename)
	end
	-- HACK: This is a weird workaround, to get util functions to refer to our environment
	local hack_functions = {
		"add_automation_script",
		"add_itemdrop_counter",
		"__raw_add_warning",
		"__raw_add_extra_warning",
		"add_ascension_zone_check",
		"add_aftercore_zone_check",
		"add_always_zone_check",
		"add_extra_ascension_warning",
		"add_ascension_warning",
		"add_extra_always_warning",
		"add_always_warning",
		"add_ascension_warning",
		"add_aftercore_warning",
		"add_always_adventure_warning",
		"add_ascension_adventure_warning",
		"add_extra_ascension_adventure_warning",
		"add_aftercore_adventure_warning",
	}
	for x in table.values(hack_functions) do
		env[x] = _G[x]
	end
	for x in table.values(hack_functions) do
		setfenv(_G[x], env)
	end

	setmetatable(env, { __index = function(t, k)
		if f_notfound then
			f_notfound(t, k, filename)
-- 			print("f_notfound", t, k, _G[k])
			return _G[k]
		end
	end})
-- 	setmetatable(env, { __index = _G })
-- 	print("runfile", filename)
	local f, e = loadfile("scripts/" .. filename)
	if not f then error(e, 2) end
	setfenv(f, env)

	f()
end

function load_script_files(env)
	local function load_file(category, name)
		if category ~= "chat" then
			local warn = true
			run_file_with_environment(name, env, function (t, k, filename)
				if warn then
					print("Warning: using global variable", k, "in", filename)
-- 					print(debug.traceback())
-- 					error("Warning: invalid __index for name '" .. tostring(k) .. "' in '" .. tostring(filename) .. "'")
				end
			end)
			warn = false
		end
	end

	run_file_with_environment("loaders.lua", { load_file = load_file })
	run_file_with_environment("user-loaders.lua", { load_file = load_file })
end


function make_kol_html_frame(contents, title, bgcolor)
	return [[<center><table  width=95%  cellspacing=0 cellpadding=0><tr><td style="color: white;" align=center bgcolor=]] .. (bgcolor or "green") .. [[><b>]] .. (title or "Results:") .. [[</b></td></tr><tr><td style="padding: 5px; border: 1px solid ]]..(bgcolor or "green")..[[;"><center><table><tr><td>]] .. contents .. [[</td></tr></table></center></td></tr><tr><td height=4></td></tr></table>]]
end

function add_formatted_colored_message_to_page(pagetext, msg, color)
	local pre_, dv_, mid_, end_, post_ = pagetext:match("^(.+)(<div style='overflow: auto'><center><table)(.+)(</body></html>)(.*)$")
	if pre_ and dv_ and mid_ and end_ and post_ then
		local wrappedmsg = [[<center><table width=95%><tr><td><p style="color: ]] .. color .. [[">]] .. msg .. [[</p></td></tr></table></center>]]
		return pre_ .. wrappedmsg .. "<br>" .. dv_ .. mid_ .. end_ .. post_
	else
		return [[<p style="color: ]] .. color .. [[">]] .. msg .. [[</p>]] .. pagetext
	end
end

function add_colored_message_to_page(pagetext, msg, color)
	local pre_, dv_, mid_, end_, post_ = pagetext:match("^(.+)(<div style='overflow: auto'><center><table)(.+)(</body></html>)(.*)$")
	if pre_ and dv_ and mid_ and end_ and post_ then
		local wrappedmsg = [[<center><table width=95%><tr><td><pre style="color: ]] .. color .. [[">]] .. msg .. [[</pre></td></tr></table></center>]]
		return pre_ .. wrappedmsg .. "<br>" .. dv_ .. mid_ .. end_ .. post_
	else
		return [[<center><pre style="color: ]] .. color .. [[">]] .. msg .. [[</pre></center>]] .. pagetext
	end
end



function run_functions(p, pagetext, run)
	original_page_text = pagetext
	pagetext = run(p, pagetext)

	if p == "/fight.php" then
		pagetext = pagetext:gsub([[(<td[^>]-><img src="http://images.kingdomofloathing.com/itemimages/)([^"]+.gif)(" width=30 height=30 alt="[^"]+" title=")([^"]+)("></td><td[^>]->)(.-)(</td></tr>)]], function (pre, itemimage, mid, title, td, msg, post)
	--~ 		print("item use", msg)
			item_image = itemimage
			item_name = title
			msg = run("used combat item", msg)
			item_image = nil
			item_name = nil
			return pre .. itemimage .. mid .. title .. td .. msg .. post
		end)
	end

	pagetext = pagetext:gsub([[<center><table class="item" style="float: none" rel="[^"]*"><tr><td><img src="http://images.kingdomofloathing.com/itemimages/[^"]+.gif" alt="[^"]*" title="[^"]*" class=hand onClick='descitem%([0-9]+%)'></td><td valign=center class=effect>You acquire .-</td></tr></table></center>]], function (droptext)
-- 			print("item drop gsub", droptext)
		item_image = droptext:match([[src="http://images.kingdomofloathing.com/itemimages/([^"]+).gif"]])
		item_name = droptext:match([[title="([^"]*)"]])
		msg = droptext
		if item_name then
			msg = run("item drop: " .. item_name, msg)
		end
		msg = run("item drop", msg)
-- 		print("item capture", pre, rel, mid, dropinfo, post)
-- 		local msg = pre .. rel .. mid .. "<span style=\"color: darkgreen\">" .. dropinfo .. "</span>" .. post
-- 		if string.match(rel, "u=u") then
-- 			msg = pre .. rel .. mid .. "<span style=\"color: darkgreen\">" .. dropinfo .. "</span> [use]" .. post
-- 			http://localhost:18481/inv_use.php?pwd=xxx&which=3&whichitem=3236
-- 		elseif string.match(rel, "u=q") then
-- 			msg = pre .. rel .. mid .. "<span style=\"color: darkgreen\">" .. dropinfo .. "</span> [equip]" .. post
-- 			http://localhost:18481/inv_equip.php?pwd=xxx&which=2&action=equip&whichitem=2813
-- 		end
		return msg
	end)

	if p == "/fight.php" then
		pagetext = pagetext:gsub([[(<!%-%-familiarmessage%-%-><center><table>.-</table></center>)]], function (msg)
			familiarmessage_picture = msg:match([[<!%-%-familiarmessage%-%-><center><table><tr><td align=center valign=center><img src="http://images.kingdomofloathing.com/itemimages/([^"]+).gif" width=30 height=30></td>]])
			if familiarmessage_picture then
				msg = run("familiar message: " .. familiarmessage_picture, msg)
			end
			msg = run("familiar message", msg)
			return msg
		end)
	end

	pagetext = run("all pages", pagetext)

	return pagetext
end
