--v1.4a-modded

local function formatStat(name, value, mult)
	if value == "" or value == nil then
		return ""
	end
		
	if mult then
		value = math.max(math.floor(value * mult), 1) -- is this floored?
	end
	return "<div><span style='margin-right:5px;'>" .. name .. ":</span><span>" .. value .. "</span></div>"
end

local function formatMonsterItems(monster)
	local data = monster.Items
	if not data then return " " end

	local itemdatalist = {}
	for value in table.values(data) do
		local dropinfo = ""
		
		if value.UnknownDropRate then
			dropinfo = "??"
		else 
			dropinfo = value.Chance .. "%"
		end
		
		if value.Conditional then
			dropinfo = dropinfo .. " (conditional)"
		end
		
		if value.PickPocketOnly then
			dropinfo = dropinfo .. " (pickpocket only)"
		end
		
		if value.NoPickPocket then
			dropinfo = dropinfo .. " (no pickpocket)"
		end
			
		if value.Bounty then
			dropinfo = dropinfo .. " (bounty)"
		end
			
		local itemData = "<div>"
		itemData = itemData .. "<span style='margin-right:5px;'>"..value.Name .. ":</span>"
		itemData = itemData .. "<span>" .. dropinfo .. "</span></div>"
		table.insert(itemdatalist, itemData)
	end
	local itemDataText = "<div style='float:left;'>"
	itemDataText = itemDataText .. "<div style='font-size:12px;'>Drops</div>"
	itemDataText = itemDataText .. table.concat(itemdatalist)
	itemDataText = itemDataText .. "</div>"
	return itemDataText
end

local function formatMonsterStats(monster)
	local data = monster.Stats
	if not data then return " " end
	local statData = "<div style='margin-right:10px;float:left;'>"
	statData = statData .. "<div style='font-size:12px;'>Stats</div>"
	statData = statData .. formatStat("HP", data.HP)
	statData = statData .. formatStat("Atk", data.Atk)
	statData = statData .. formatStat("Def", data.Def, 0.9)
	statData = statData .. formatStat("Meat", data.Meat)
	statData = statData .. formatStat("Element", data.Element)
	statData = statData .. formatStat("Init", data.Init)
	statData = statData .. formatStat("Phylum", data.Phylum)
	statData = statData .. formatStat("PhyRes", data.PhyRes)
	statData = statData .. formatStat("Watch out for", data.WatchOut)
	statData = statData .. "</div>"
	return statData
end

--TODO: Resolve as many ambiguous names as you can find
-- orcish frat boy
-- ninja snowman
-- slimetube
-- hobopolis 
-- more...
local ambiguousNames = {
	[ [[animated nightstand]] ] = { 
		[ [[darkstand.gif]] ] = [[animated nightstand (mahogany)]]
	}
}

local function parseMonsterNameIntoMafiaFormat(monsterName, fightText)
	--mafia's table doesn't precede the names of the monsters
	--with "a " or "an " or spaces, this function attempts to
	--strip out those things
	--mafia's table was also inconsistent with cases, 
	--so I lowercased everything and will have to do so here as well
	
	local mafiaMonsterName = monsterName:lower()

	mafiaMonsterName = mafiaMonsterName:gsub("^a ", "")
	mafiaMonsterName = mafiaMonsterName:gsub("^an ", "")
	mafiaMonsterName = mafiaMonsterName:gsub("^ ", "")

	if ambiguousNames[monsterName] then
		for lookFor, actualName in pairs(ambiguousNames[monsterName]) do
			if fightText:contains(lookFor) then
				mafiaMonsterName = actualName
			end
		end	
	end

	return mafiaMonsterName
end

local function getMonsterData(monsterName, fightText)
	local mlmods = estimate_ML_modifiers()
	local ml = 0
	for _, m in pairs(mlmods) do
		ml = ml + m
	end

	local itemmods = estimate_item_modifiers()
	local item = 0
	for _, m in pairs(itemmods) do
		item = item + m
	end

	local monsterDataName = parseMonsterNameIntoMafiaFormat(monsterName, fightText)
	local base_data = get_monster_base_data(monsterDataName)
	local ml_increases = {
		HP = true,
		Atk = true,
		Def = true,
	}
	if base_data then
		local monster = {}
		if base_data.Stats then
			monster.Stats = {}
			for a, b in pairs(base_data.Stats) do
				if ml_increases[a] then
					monster.Stats[a] = math.max(b + ml, 1)
				else
					monster.Stats[a] = b
				end
			end
		end
		if base_data.Items then
			monster.Items = {}
			for idata in table.values(base_data.Items) do
				local newidata = {}
				for a, b in pairs(idata) do
					if a == "Chance" then
						local p_base = b / 100
						local p_mod = p_base * (1 + item / 100)
						p_mod = math.max(math.min(p_mod, 1), 0)
						newidata[a] = round_down(p_mod * 100, 2)
					else
						newidata[a] = b
					end
				end
				table.insert(monster.Items, newidata)
			end
		end
		return monster
	else
		return nil
	end
end

add_printer("/fight.php", function()
	if character["show monster stats"] == "yes" then
		local monster = getMonsterData(monster_name, text)
		if monster then
			text = text:gsub([[<td><img id='monpic']], [[<td rowspan=2><img id='monpic']])	
			text = text:gsub([[(id='monname'.-)</td></tr></table><br>]], function(prefix)
				return prefix .. [[</td></tr><tr><td><div style='font-size:11px;color:#555;'>]] .. formatMonsterStats(monster) .. formatMonsterItems(monster) .. [[</div></td></tr></table><br>]]
			end)
		end
	end
end)

add_processor("/fight.php", function()
	if text:contains("You slap a flyer up on your opponent") then
		local flyertext = text:match([[documents.gif" width=30 height=30 alt=.-You slap a flyer up on your opponent]])
		local whichflyer = flyertext:match([[title="(.-)"]])
		if whichflyer then
			local monster = getMonsterData(monster_name, text)
			if monster and monster.Stats and monster.Stats.Atk then
				if whichflyer == "rock band flyers" then
					increase_ascension_counter("zone.island.frat arena flyerML", monster.Stats.Atk)
				elseif whichflyer == "jam band flyers" then
					increase_ascension_counter("zone.island.hippy arena flyerML", monster.Stats.Atk)
				end
			end
		end
	end
end)

add_processor("/fight.php", function()
	--TODO: Count fight damage to monster
end)

add_printer("/fight.php", function()
	if text:contains("You slap a flyer up on your opponent") then
		text = text:gsub([[documents.gif" width=30 height=30 alt=.-</td></tr>]], function(flyertext)
			local completion = "?"
			local whichband = "?"
			if flyertext:contains([[title="rock band flyers"]]) then
				completion = round_down((ascension["zone.island.frat arena flyerML"] or 0) / 100, 1)
				whichband = "rock band"
			elseif flyertext:contains([[title="jam band flyers"]]) then
				completion = round_down((ascension["zone.island.hippy arena flyerML"] or 0) / 100, 1)
				whichband = "jam band"
			end
			return flyertext:gsub([[(You slap a flyer up on your opponent.-)(<)]], [[%1 <span style="color:green">{ Advertised ~]] .. completion .. [[%% for ]]..whichband..[[. }</span>%2]])
		end)
	end
end)

add_printer("/bigisland.php", function()
	if have("rock band flyers") or have("jam band flyers") then
		local msgs = {}
		
		if have("rock band flyers") then
			table.insert(msgs, "{ ~" .. (ascension["zone.island.frat arena flyerML"] or 0) .. " ML slapped for frat boys. }")
		end
		if have("jam band flyers") then
			table.insert(msgs, "{ ~" .. (ascension["zone.island.hippy arena flyerML"] or 0) .. " ML slapped for hippies. }")
		end
		text = text:gsub([[<p><Center>]], [[<p style="color:green;text-align:center;">]] .. table.concat(msgs, "<br>\n") .. [[</p>%0]])
	end
end)
